"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bodyguard_1 = require("../bodyguard");
var enums_1 = require("../enums");
/**
 * Get readable relations for a payload, given a User
 * @param payload Payload object to check
 * @param user User to authorize fields for
 * @param objAlias Object alias from a SQL query to prepend
 * 	to the member keys
 * @return Returns an array of field names
 */
function getReadableRelations(payload, user, objAlias) {
    var readableFields = [];
    // Traverse members of payload, and append the member to
    // readable fields if the CanRead() key allows
    for (var member in payload) {
        if (member && !(payload[member] instanceof Function)) {
            var canRead = bodyguard_1.getCanReadRelation(payload, member);
            // Check if the key has a CanRead() decorator, and if it equals Anyone
            // Exceptions:
            // 	a. The CanRead() key is a UserRole that is equivalent to the User's role
            //  b. The CanRead() key is set to Self and the user is authenticated (we'll filter later)
            // 	c. The CanRead() key is set to Admin and the user is an admin
            if (canRead && // must have CanRead() key
                (canRead === enums_1.BodyguardOwner.Anyone || // Anyone
                    (user && canRead === user.role) || // By role
                    (user && canRead === enums_1.BodyguardOwner.Self && user.id) || // Self
                    (user && // Admin
                        canRead === enums_1.BodyguardOwner.Admin &&
                        user.role === enums_1.UserRole.Admin))) {
                readableFields.push(objAlias ? objAlias + '.' + member : member);
            }
        }
    }
    // Return readable fields
    return readableFields;
}
exports.getReadableRelations = getReadableRelations;
//# sourceMappingURL=get-readable-relations.js.map