"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bodyguard_1 = require("../bodyguard");
var utils_1 = require("../utils");
var enums_1 = require("../enums");
/**
 * Filter an outgoing response body to ensure it doesn't leak private fields
 * @param obj Object or array to check (enables recursion).
 * 	This can be the direct obj set of a typeorm request
 * @param user User object to check against
 * @param objType Type of object to check
 * @param userType Type of User to check
 * @return Returns the filtered obj
 */
function readFilter(obj, user, objType, userType) {
    if (obj instanceof Array) {
        for (var i = 0; i < obj.length; i++) {
            var subObjType = obj[i].constructor;
            obj[i] = readFilter(obj[i], user, subObjType, userType);
        }
    }
    else if (obj instanceof Object) {
        var isSelfResult = utils_1.isSelf(obj, user, objType, userType);
        var isAdminResult = utils_1.isAdmin(user);
        // Loop through object members
        for (var member in obj) {
            if (!(obj[member] instanceof Function)) {
                if (obj[member] instanceof Array ||
                    (obj[member] instanceof Object &&
                        !(obj[member] instanceof Date))) {
                    var subObjType = obj[member].constructor;
                    if (subObjType !== {}.constructor) {
                        obj[member] = readFilter(obj[member], user, subObjType, userType);
                    }
                }
                else if (member !== 'id') {
                    var canRead = bodyguard_1.getCanRead(new objType(), member);
                    if (canRead === undefined) {
                        delete obj[member];
                    }
                    else if (canRead !== enums_1.BodyguardOwner.Anyone &&
                        ((canRead === enums_1.BodyguardOwner.Self && !isSelfResult) ||
                            (canRead === enums_1.BodyguardOwner.Admin &&
                                !isAdminResult))) {
                        delete obj[member];
                    }
                }
            }
        }
    }
    return obj;
}
exports.readFilter = readFilter;
//# sourceMappingURL=read-filter.js.map