"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bodyguard_1 = require("../bodyguard");
var utils_1 = require("../utils");
var query_types_1 = require("../query-tools/query-types");
var enums_1 = require("../enums");
/**
 * Get Filter: Filter a GET response to remove private fields
 * @param request Request object to query by
 * @param response Response object to call responder with
 * @param next Next function to call on success
 */
function getFilter(request, response, next) {
    var denied = false;
    // Check incoming
    if (request.query) {
        var isAdminResult = utils_1.isAdmin(request.user);
        // Loop through object members
        var field = void 0;
        for (var _i = 0, queryTypeKeys_1 = query_types_1.queryTypeKeys; _i < queryTypeKeys_1.length; _i++) {
            field = queryTypeKeys_1[_i];
            // Skip joins
            if (field === 'join' || field === 'additionalParameters') {
                continue;
            }
            field = request.query[field];
            if (field instanceof Array) {
                for (var _a = 0, field_1 = field; _a < field_1.length; _a++) {
                    var member = field_1[_a];
                    if (member.includes('.')) {
                        member = member.split('.')[0];
                    }
                    if (!(field[member] instanceof Function) &&
                        member.indexOf('__') !== 0) {
                        // Get read privileges for the field
                        var canRead = bodyguard_1.getCanRead(new request.payloadType(), member);
                        if (canRead === undefined ||
                            (canRead !== enums_1.BodyguardOwner.Anyone &&
                                ((canRead === enums_1.BodyguardOwner.Self &&
                                    !request.user) ||
                                    (canRead === enums_1.BodyguardOwner.Admin &&
                                        !isAdminResult)))) {
                            // User does not have privilege
                            denied = member;
                        }
                    }
                }
            }
            else if (field instanceof Object) {
                for (var member in field) {
                    if (member.includes('.')) {
                        member = member.split('.')[0];
                    }
                    if (!(field[member] instanceof Function) &&
                        member.indexOf('__') !== 0) {
                        // Get read privileges for the field
                        var canRead = bodyguard_1.getCanRead(new request.payloadType(), member);
                        if (canRead === undefined ||
                            (canRead !== enums_1.BodyguardOwner.Anyone &&
                                ((canRead === enums_1.BodyguardOwner.Self &&
                                    !request.user) ||
                                    (canRead === enums_1.BodyguardOwner.Admin &&
                                        !isAdminResult)))) {
                            // User does not have privilege
                            denied = member;
                        }
                    }
                }
            }
        }
    }
    if (denied) {
        // Cannot get by member, issue 403 Forbiddenn
        response.forbiddenResponder('Cannot get by member ' + denied);
    }
    else {
        // User is authorized, filter response payload
        request.payload = bodyguard_1.readFilter(request.payload, request.user, request.payloadType, request.userType);
        // Proceed
        next();
    }
}
exports.getFilter = getFilter;
//# sourceMappingURL=get-filter.js.map