"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var base_user_1 = require("../models/base-user");
var user_status_1 = require("../enums/user-status");
var is_admin_1 = require("../utils/is-admin");
/**
 * Only Actie Guard: Check if the user has a status
 *	and that it matches UserStatus.Active, otherwise
 *	respond with 403 Forbidden
 * @param request Request object to query by
 * @param response Response object to call responder with
 * @param next Next function to call on success
 */
function onlyActive(request, response, next) {
    if (request.user instanceof base_user_1.BaseUser) {
        if (request.user.status === user_status_1.UserStatus.Active ||
            is_admin_1.isAdmin(request.user)) {
            // User is active
            next();
        }
        else {
            // User is not active, respond with 403 Forbidden
            response.forbiddenResponder('not active');
        }
    }
    else {
        // User is not authenticated, respond with 401 Unauthorized
        response.unauthorizedResponder('not authenticated');
    }
}
exports.onlyActive = onlyActive;
//# sourceMappingURL=only-active.js.map