"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var base_user_1 = require("../models/base-user");
var user_role_1 = require("../enums/user-role");
var is_admin_1 = require("../utils/is-admin");
/**
 * Only allow user with the role Member (or Admin+),
 * 	or respond with 403 Forbidden
 * @param request Request object to query by
 * @param response Response object to call responder with
 * @param next Next function to call on success
 */
function onlyMember(request, response, next) {
    if (request.user instanceof base_user_1.BaseUser) {
        if (request.user.role === user_role_1.UserRole.Member || is_admin_1.isAdmin(request.user)) {
            // User is member
            next();
        }
        else {
            // User is not member, respond with 403 Forbidden
            response.forbiddenResponder('not member');
        }
    }
    else {
        // User is not authenticated, respond with 401 Unauthorized
        response.unauthorizedResponder('not member');
    }
}
exports.onlyMember = onlyMember;
//# sourceMappingURL=only-member.js.map