"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var base_user_1 = require("../models/base-user");
var user_role_1 = require("../enums/user-role");
var bodyguard_1 = require("../bodyguard");
var utils_1 = require("../utils");
var base_model_1 = require("../models/base-model");
/**
 * Only Self Guard: Return 403 Forbidden if the User does not
 * 	own the resource
 * @param request Request object to query by
 * @param response Response object to call responder with
 * @param next Next function to call on success
 */
function onlySelf(request, response, next) {
    var authorized = false;
    var userKeys = bodyguard_1.getBodyguardKeys(new request.userType());
    var bodyKeys = bodyguard_1.getBodyguardKeys(new request.payloadType());
    if (request.user instanceof base_user_1.BaseUser) {
        if (request.user.role === user_role_1.UserRole.Admin) {
            authorized = true;
        }
        else {
            // Create
            if (request.method === 'POST') {
                authorized = utils_1.isSelf(request.body, request.user, request.payloadType, request.userType, bodyKeys, userKeys);
            }
            else if (request.method === 'GET') {
                // Read
                if (request.identifier in request.query &&
                    request.payload instanceof base_model_1.BaseModel) {
                    // Query by id
                    authorized = utils_1.isSelf(request.payload, request.user, request.payloadType, request.userType, bodyKeys, userKeys);
                }
                else if (request.payload instanceof Array) {
                    // Filter payload
                    request.payload = request.payload.filter(function (result) {
                        return utils_1.isSelf(result, request.user, request.payloadType, request.userType, bodyKeys, userKeys);
                    });
                    authorized = true;
                }
            }
            else if ((request.method === 'PATCH' || request.method === 'PATCH') &&
                request.payload instanceof base_model_1.BaseModel) {
                // Update
                authorized = utils_1.isSelf(request.payload, request.user, request.payloadType, request.userType, bodyKeys, userKeys);
            }
            else if (request.method === 'DELETE' &&
                request.payload instanceof base_model_1.BaseModel) {
                // Delete
                authorized = utils_1.isSelf(request.payload, request.user, request.payloadType, request.userType, bodyKeys, userKeys);
            }
        }
    }
    else {
        // User is not authenticated, respond with 401 Unauthorized
        response.unauthorizedResponder('not self');
        return false;
    }
    if (authorized) {
        // User is self
        next();
    }
    else {
        // User is not authenticated or self, respond with 403 Forbidden
        response.forbiddenResponder('not self');
        return false;
    }
}
exports.onlySelf = onlySelf;
//# sourceMappingURL=only-self.js.map