/**
 * # HttpClient
 *
 * HttpClient can be used to mock requests in your test-suite. There are
 * 4 functions: post(), get(), patch(), delete()
 *
 * **Before using, set HttpClient.url and HttpClient.port**
 *
 */
/**
 * HttpClient
 */
import * as request from 'request';
import { HttpClientResponse } from './http-client-response';
export declare class HttpClient {
    request: request.RequestAPI<request.Request, request.CoreOptions, request.RequiredUriUrl>;
    url: string;
    port: number | string;
    headers?: Object;
    /**
     * Send a POST http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    post(path: string, data: object, bearer?: boolean | string, expect?: number[], customOptions?: object): Promise<HttpClientResponse>;
    /**
     * Send a GET http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    get(path: string, data?: boolean | Object, bearer?: boolean | string, expect?: number[], customOptions?: object): Promise<HttpClientResponse>;
    /**
     * Send a PATCH http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    patch(path: string, data: object, bearer?: boolean | string, expect?: number[], customOptions?: object): Promise<HttpClientResponse>;
    /**
     * Send a DELETE http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    delete(path: string, bearer?: boolean | string, expect?: number[], customOptions?: object): Promise<HttpClientResponse>;
}
