/**
 * # JWT Tokens
 *
 * JWT Authorization headers are the default way to handle authentication.
 *
 * If you use the loginEndpoint and onlySelf/onlyAdmin guards, this will
 * be taken care of for you.
 */
/**
 * JwtBearer
 */
import { BaseUser } from './models/base-user';
import { Request } from 'express';
/**
 * JWT Bearer for authentication
 */
export declare class JwtBearer {
    key: string;
    /**
     * Construct a JWT Bearer token
     * @param key Private key
     */
    constructor(key?: string);
    /**
     * Get expiration of the token
     * @return Returns the expiration epoch time
     */
    getExpiration(isRefresh?: boolean): number;
    /**
     * Sign the token to the User
     * @param user User to sign the JWT for
     * @return Returns the signed, base64-encoded token
     */
    sign(user: BaseUser, isRefresh?: boolean, data?: Object): string;
    /**
     * Get a token from an HTTP Request
     * @param request Request to fetch the token from
     * @return Returns the token if successful, otherwise false
     */
    getToken(request: Request): boolean | string;
    /**
     * Verify the token
     * @param token Token to verify
     * @return Returns the token payload, or false if invalid
     */
    dryVerify(token: string): any;
}
export declare const jwtBearer: JwtBearer;
