"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Basic CORS Middleware
 * @param request Express request
 * @param response Express response
 * @param next Next function to call on success
 */
function basicCors(request, response, next) {
    // Set Access-Control-Allow-Origin
    var origin = '*';
    // Backwards compatability
    // TODO: Remove in v3.0.0
    if (!('ALLOW_ORIGIN' in process.env) && 'CLIENT_URL' in process.env) {
        process.env.ALLOW_ORIGIN = process.env.CLIENT_URL;
    }
    if ('ALLOW_ORIGIN' in process.env && process.env.ALLOW_ORIGIN) {
        if (process.env.ALLOW_ORIGIN.includes(', ')) {
            // Array of Client URLs
            var host = request.headers.origin;
            var clientUrls = process.env.ALLOW_ORIGIN.split(', ');
            if (host instanceof Array) {
                host = host[0];
            }
            if (clientUrls.includes(host)) {
                origin = host;
            }
            else {
                origin = clientUrls[0];
            }
        }
        else if (typeof process.env.ALLOW_ORIGIN === 'string') {
            // Single Client Url
            origin = process.env.ALLOW_ORIGIN;
        }
    }
    response.setHeader('Access-Control-Allow-Origin', origin);
    response.setHeader('Access-Control-Allow-Methods', 'POST, GET, PATCH, DELETE, OPTIONS');
    response.setHeader('Access-Control-Allow-Headers', 'X-Requested-With, Content-Type, Authorization, Cache-Control');
    response.setHeader('Access-Control-Allow-Credentials', 'true');
    response.setHeader('Content-Type', 'application/json;charset=utf-8');
    next();
}
exports.basicCors = basicCors;
//# sourceMappingURL=basic-cors.js.map