"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
// Express
var express = require('express');
var bodyParser = require("body-parser");
// Pointy Core
var listen_1 = require("./utils/listen");
var database_1 = require("./database");
var http_client_1 = require("./http/http-client");
// Handlers
var responders_1 = require("./responders");
var handlers_1 = require("./handlers");
var bind_responders_1 = require("./utils/bind-responders");
// Base Models
var models_1 = require("./models");
/**
 * PointyAPI App Instance
 */
var PointyApi = /** @class */ (function () {
    // Initialize
    function PointyApi() {
        var _this = this;
        // Express app
        this.app = express();
        // Core
        this.listen = listen_1.listen;
        this.http = new http_client_1.HttpClient();
        this.userType = models_1.ExampleUser;
        this.testmode = false;
        // Middleware
        // Handlers
        this.log = handlers_1.logHandler;
        this.error = handlers_1.errorHandler;
        // Responders
        this.conflictResponder = responders_1.conflictResponder;
        this.forbiddenResponder = responders_1.forbiddenResponder;
        this.goneResponder = responders_1.goneResponder;
        this.unauthorizedResponder = responders_1.unauthorizedResponder;
        this.validationResponder = responders_1.validationResponder;
        this.deleteResponder = responders_1.deleteResponder;
        this.getResponder = responders_1.getResponder;
        this.postResponder = responders_1.postResponder;
        this.patchResponder = responders_1.patchResponder;
        // Hooks
        this.before = function (app) { };
        this.ready = function (app) { };
        // Default db
        this.db = new database_1.PointyPostgres();
        this.db.logger = this.log;
        if (process.argv.includes('testmode')) {
            this.testmode = true;
        }
        this.app.on('error', function (error) { return _this.error(error); });
    }
    // Ready check
    PointyApi.prototype.readycheck = function () {
        // Check pointy.userType
        if (!this.userType) {
            console.warn('[PointyAPI] pointy.userType has not been set!');
            return false;
        }
        // Check database connection
        if (!this.db || !this.db.conn) {
            console.warn('[PointyAPI] Database connection has not been established');
            return false;
        }
        // Check database entities
        if (!this.db.entities || !this.db.entities.length) {
            console.warn('[PointyAPI] Database does not contain any entities');
            return false;
        }
        // Check for database entity of BaseUser
        if (this.db.entities.includes(models_1.BaseUser)) {
            console.warn('[PointyAPI] BaseUser is not a valid database entity');
            return false;
        }
        // Checks passed
        return true;
    };
    // Start
    PointyApi.prototype.start = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // Set proper headers
                        this.app.disable('x-powered-by');
                        // Apply middleware functions
                        this.app.use(function (request, response, next) {
                            request.userType = _this.userType;
                            request.joinMembers = [];
                            // Bind the request & response to the responders
                            bind_responders_1.bindResponders(_this, request, response);
                            next();
                        });
                        // Use body-parser
                        this.app.use(bodyParser.json());
                        this.app.use(bodyParser.urlencoded({ extended: true }));
                        // Run before function
                        return [4 /*yield*/, this.before(this.app)];
                    case 1:
                        // Run before function
                        _a.sent();
                        // Ready check
                        if (this.readycheck()) {
                            // Server listen
                            this.listen(this.app, process.env.PORT, this.log);
                            // Send IPC notification
                            if ('send' in process && process.send) {
                                process.send('server-ready');
                            }
                            // Run ready callback
                            this.ready(this.app);
                        }
                        else {
                            process.exit();
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    return PointyApi;
}());
exports.PointyApi = PointyApi;
exports.pointy = new PointyApi();
//# sourceMappingURL=pointy-core.js.map