"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var class_validator_1 = require("class-validator");
/**
 * Get class-validator constraints for a class
 * @param someClass BaseModel class(e.g `request.payloadType`)
 * @param key (Optional) Key to get constraints for. If unset, constraints for
 * 	all keys will be returned.
 */
function getValidationConstraints(someClass, key) {
    var container = class_validator_1.getFromContainer(class_validator_1.MetadataStorage);
    var metadata = container.getTargetValidationMetadatas(someClass, JSON.stringify(someClass));
    var properties = container.groupByPropertyName(metadata);
    var validators = Object.keys(properties).reduce(function (schema, property) {
        schema[property] = properties[property].reduce(function (propertySchema, _a) {
            var type = _a.type, constraints = _a.constraints;
            if (Array.isArray(constraints) && constraints.length === 1) {
                constraints = constraints[0];
            }
            if (typeof constraints === 'undefined') {
                propertySchema[type] = true;
            }
            else {
                propertySchema[type] = constraints;
            }
            return propertySchema;
        }, {});
        return schema;
    }, {});
    if (key) {
        if (key in validators) {
            return validators[key];
        }
        else {
            return false;
        }
    }
    else {
        return validators;
    }
}
exports.getValidationConstraints = getValidationConstraints;
//# sourceMappingURL=get-validation-constraints.js.map