/**
 * # Set Model
 *
 * Set model sets the entity the request should use. You can also
 * set the route parameter (e.g. ':id'). This function should
 * be called in a middleware function BEFORE your endpoints.
 *
 * ## Auth Routes
 *
 * Auth routes must specify to `setModel` that it is an auth route. Do
 * this by passing `true` to the `isAuth` (fourth) parameter.
 */
/**
 * Set Model
 */
import { Request, Response } from 'express';
import { BaseModelInterface } from './models';
/**
 * Set model type and load payload
 * @param request Request object
 * @param response Response object
 * @param model Model to set the request type to
 * @param isAuth If this is an authentication route.
 * 	Default is false.
 * 	Will run beforeLogin() and beforeLogout() posts instead of post/delete hook
 * @param identifier URL parameter name, for
 * 	example `/users/:id`. Default is `id`. Although this parameter
 * 	is optional, you **must** set it to the same string as in your routes.
 * @return Returns a Promise
 */
export declare function setModel(request: Request, response: Response, model: BaseModelInterface, isAuth?: boolean, identifier?: string): Promise<any>;
