"use strict";
/**
 * # Set Model
 *
 * Set model sets the entity the request should use. You can also
 * set the route parameter (e.g. ':id'). This function should
 * be called in a middleware function BEFORE your endpoints.
 *
 * ## Auth Routes
 *
 * Auth routes must specify to `setModel` that it is an auth route. Do
 * this by passing `true` to the `isAuth` (fourth) parameter.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var typeorm_1 = require("typeorm");
var middleware_1 = require("./middleware");
var utils_1 = require("./utils");
var query_validator_1 = require("./query-tools/query-validator");
/**
 * Set model type and load payload
 * @param request Request object
 * @param response Response object
 * @param model Model to set the request type to
 * @param isAuth If this is an authentication route.
 * 	Default is false.
 * 	Will run beforeLogin() and beforeLogout() posts instead of post/delete hook
 * @param identifier URL parameter name, for
 * 	example `/users/:id`. Default is `id`. Although this parameter
 * 	is optional, you **must** set it to the same string as in your routes.
 * @return Returns a Promise
 */
function setModel(request, response, model, isAuth, identifier) {
    if (isAuth === void 0) { isAuth = false; }
    if (identifier === void 0) { identifier = 'id'; }
    return __awaiter(this, void 0, void 0, function () {
        var i, key, key, getSuccess_1, _a, key;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    request.identifier = identifier;
                    request.payloadType = model;
                    request.payload = new model();
                    request.repository = typeorm_1.getRepository(request.payloadType);
                    // Substitute authenticated user as resource for auth router deletes
                    if (isAuth && request.method === 'DELETE') {
                        if (request.user) {
                            request.params = request.user;
                        }
                        else {
                            response.unauthorizedResponder('Not authenticated');
                            return [2 /*return*/, false];
                        }
                    }
                    if (!(request.method === 'POST')) return [3 /*break*/, 2];
                    // Load post array
                    if (request.body instanceof Array) {
                        for (i = 0; i < request.body.length; i++) {
                            request.body[i] = Object.assign(new request.payloadType(), request.body[i]);
                            for (key in request.body[i]) {
                                if (request.body[i][key] === undefined) {
                                    delete request.body[i][key];
                                }
                                if (!utils_1.isKeyInModel(key, request.payload, response)) {
                                    return [2 /*return*/, false];
                                }
                            }
                        }
                    }
                    else {
                        // Load post object
                        request.body = Object.assign(new request.payloadType(), request.body);
                        for (key in request.body) {
                            if (request.body[key] === undefined) {
                                delete request.body[key];
                            }
                            if (!utils_1.isKeyInModel(key, request.payload, response)) {
                                return [2 /*return*/, false];
                            }
                        }
                    }
                    return [4 /*yield*/, utils_1.runHook(isAuth ? 'beforeLogin' : 'beforePost', request.body, request, response)];
                case 1:
                    // Run model hook
                    if (!(_b.sent())) {
                        return [2 /*return*/, false];
                    }
                    return [3 /*break*/, 10];
                case 2:
                    if (!(request.method === 'GET')) return [3 /*break*/, 4];
                    // Get loader
                    if (!query_validator_1.queryValidator(request, response)) {
                        return [2 /*return*/, false];
                    }
                    getSuccess_1 = true;
                    _a = request;
                    return [4 /*yield*/, middleware_1.getQuery(request, response).catch(function (error) {
                            if (!response.headersSent) {
                                response.error(error);
                            }
                            getSuccess_1 = false;
                        })];
                case 3:
                    _a.payload = _b.sent();
                    if (!getSuccess_1) {
                        return [2 /*return*/, false];
                    }
                    return [3 /*break*/, 10];
                case 4:
                    if (!(request.method === 'PATCH')) return [3 /*break*/, 7];
                    // Patch loader
                    for (key in request.body) {
                        if (!utils_1.isKeyInModel(key, request.payload, response)) {
                            return [2 /*return*/, false];
                        }
                    }
                    return [4 /*yield*/, middleware_1.loadEntity(request, response)];
                case 5:
                    // Load entity
                    if (!(_b.sent())) {
                        return [2 /*return*/, false];
                    }
                    // Set model type
                    request.body = Object.assign(new request.payloadType(), request.body);
                    request.body = utils_1.deleteUndefinedMembers(request.body);
                    return [4 /*yield*/, utils_1.runHook('beforePatch', request.body, request, response)];
                case 6:
                    // Run model hook
                    if (!(_b.sent())) {
                        return [2 /*return*/, false];
                    }
                    return [3 /*break*/, 10];
                case 7:
                    if (!(request.method === 'DELETE')) return [3 /*break*/, 10];
                    return [4 /*yield*/, middleware_1.loadEntity(request, response)];
                case 8:
                    // Load entity
                    if (!(_b.sent())) {
                        return [2 /*return*/, false];
                    }
                    return [4 /*yield*/, utils_1.runHook(isAuth ? 'beforeLogout' : 'beforeDelete', request.payload, request, response)];
                case 9:
                    // Delete loader
                    // Run model hook
                    if (!(_b.sent())) {
                        return [2 /*return*/, false];
                    }
                    _b.label = 10;
                case 10: return [2 /*return*/, !response.headersSent];
            }
        });
    });
}
exports.setModel = setModel;
//# sourceMappingURL=set-model.js.map