"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var mock_req_res_1 = require("mock-req-res");
var typeorm_1 = require("typeorm");
var models_1 = require("../models");
/**
 * Create mock request/response objects
 * @param method HTTP method verb for this request
 * @param url URL to mock the request
 * @return Returns { request, response }
 */
function createMockRequest(method, url) {
    if (method === void 0) { method = 'GET'; }
    if (url === void 0) { url = '/api/v1/user'; }
    // Create request
    var request = mock_req_res_1.mockRequest();
    request.repository = typeorm_1.getRepository(models_1.ExampleUser);
    request.method = method;
    request.baseUrl = url;
    request.userType = models_1.ExampleUser;
    request.payloadType = models_1.ExampleUser;
    request.joinMembers = [];
    request.identifier = 'id';
    request.header = function (key) { return (key === 'host' ? 'localhost' : false); };
    // Create response
    var response = mock_req_res_1.mockResponse();
    // Conflict responder
    response.conflictResponder = function (error) {
        return fail('Conflict: ' + JSON.stringify(error));
    };
    // Delete responder
    response.deleteResponder = function (result) {
        return fail('Delete: ' + JSON.stringify(result));
    };
    // Forbidden responder
    response.forbiddenResponder = function (error) {
        return fail('Forbidden: ' + JSON.stringify(error));
    };
    // Get responder
    response.getResponder = function (result) { return fail('Get: ' + JSON.stringify(result)); };
    // Error responder
    response.error = function (error) { return fail('Error: ' + JSON.stringify(error)); };
    // Gone responder
    response.goneResponder = function (error) { return fail('Gone: ' + JSON.stringify(error)); };
    // Post responder
    response.postResponder = function (result) {
        return fail('Post: ' + JSON.stringify(result));
    };
    // Patch responder
    response.patchResponder = function (result) {
        return fail('Patch: ' + JSON.stringify(result));
    };
    // Unauthorized responder
    response.unauthorizedResponder = function (error) {
        return fail('Unauthorized: ' + JSON.stringify(error));
    };
    // Validation responder
    response.validationResponder = function (error) {
        return fail('Validation: ' + JSON.stringify(error));
    };
    // Return request/response
    return { request: request, response: response };
}
exports.createMockRequest = createMockRequest;
//# sourceMappingURL=create-mock-request.js.map