"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
/**
 * # Fork Server
 *
 * Fork the server for testing
 *
 * ```typescript
 * import { pointy } from '../../../src/index';
 * import { forkServer } from '../../../src/fork-server';
 *
 * const http = pointy.http;
 *
 * let serverfork;
 *
 * beforeAll(async () => {
 * 	serverfork = await forkServer('./lib/test/examples/basic/server.js');
 * });
 *
 * afterAll(() => {
 * 	if (serverfork) {
 * 		serverfork.kill();
 * 	}
 * });
 *
 * describe('API Server', () => {
 * 	it('is running', async () => {
 * 		await http
 * 			.get('/', {}, undefined, [ 200, 404 ])
 * 			.catch((error) => fail(error));
 * 	});
 * });
 * ```
 *
 */
/**
 * Fork Server
 */
var child_process_1 = require("child_process");
/**
 * Fork the server from a test-suite
 * @param serverFile File path to the server entry-point
 * @return Returns a Promise
 */
function forkServer(serverFile, args) {
    if (args === void 0) { args = ['testmode']; }
    return new Promise(function (accept, reject) {
        fs.access(serverFile, function (error) {
            if (error) {
                reject('Could not read server file: ' + JSON.stringify(error));
            }
            else {
                var serverfork_1 = child_process_1.fork(serverFile, args);
                serverfork_1.on('message', function (message) {
                    accept(serverfork_1);
                });
            }
        });
    });
}
exports.forkServer = forkServer;
//# sourceMappingURL=fork-server.js.map